@@ -17,7 +17,7 @@
 # ==============================================================================
 from typing import Type, List, Union
 
-from tfaip.base.model import GraphBase
+from tfaip.base.imports import GraphBase
 from tfaip.base.model.util.module import import_graphs
 from tfaip.util.enum import StrEnum
 
