@@ -22,6 +22,7 @@ import os
 import sys
 import tempfile
 from abc import ABC, abstractmethod
+from contextlib import ExitStack
 from dataclasses import dataclass
 from dataclasses_json import dataclass_json
 import logging
@@ -30,30 +31,32 @@ import tensorflow as tf
 import tensorflow.keras as keras
 import re
 
+from tensorflow.python.keras.metrics import MeanMetricWrapper
 from tensorflow_addons.optimizers import MovingAverage
 
 from tfaip.base.data.data import DataBase
+from tfaip.base.evaluator.evaluator import Evaluator
+from tfaip.base.evaluator.params import EvaluatorParams
+from tfaip.base.lav.multilav import MultiLAV
 from tfaip.base.model.exportgraph import ExportGraph
 from tfaip.base.scenario.scenariobaseparams import ScenarioBaseParams, NetConfigParamsBase, NetConfigNodeSpec
 from tfaip.base.scenario.util.keras_debug_model import KerasDebugModel
+from tfaip.base.scenario.util.outputholder import OutputHolderMetricWrapper
 from tfaip.base.scenario.util.print_evaluate_layer import PrintEvaluateLayer
 
 from tensorflow.python.framework.convert_to_constants import convert_variables_to_constants_v2
 
+from tfaip.util.typing import AnyTensor
+
 if TYPE_CHECKING:
     from tfaip.base.trainer import TrainerParams
     from tfaip.base.model import ModelBase
     from tfaip.base.lav import LAVParams, LAV
-    from tfaip.base.predict import Predictor, PredictorParams
+    from tfaip.base.predict import Predictor, PredictorParams, MultiModelPredictor
 
 logger = logging.getLogger(__name__)
 
 
-# Label metric because it is used by keras fit as output suffix (e.g. ACC_metric)
-def metric(t, p):
-    return p
-
-
 def list_scenarios_in_module(module) -> List[Tuple[str, Type['ScenarioBase']]]:
     return inspect.getmembers(module, lambda member: inspect.isclass(member)
                                                      and member.__module__ == module.__name__
@@ -67,6 +70,7 @@ def list_scenarios_in_module(module) -> List[Tuple[str, Type['ScenarioBase']]]:
 class KerasModelData:
     loss_names: List[str]
     extended_metric_names: List[str]
+    tensorboard_output_names: List[str]
 
 
 class ScenarioBase(ABC):
@@ -94,34 +98,82 @@ class ScenarioBase(ABC):
         scenario_params.scenario_base_path_ = inspect.getfile(cls)
         scenario_params.scenario_module_ = cls.__module__
         scenario_params.model_params = cls.model_cls().get_params_cls()()
-        scenario_params.data_params = cls.data_cls().get_params_cls()()
+        scenario_params.data_params = cls.data_cls().get_default_params()
+        scenario_params.evaluator_params = cls.evaluator_cls().default_params()
         return scenario_params
 
+    @classmethod
+    def default_trainer_params(cls) -> 'TrainerParams':
+        trainer_params = cls.trainer_cls().get_params_cls()()
+        trainer_params.scenario_params = cls.default_params()
+        return trainer_params
+
     @classmethod
     def params_from_dict(cls, d: dict) -> ScenarioBaseParams:
+        if cls == ScenarioBase:
+            raise ValueError("You are calling this method from the ScenarioBaseClass. This is not supported. "
+                             "Call this method from an actual Scenario or call from_dict to obtain the real class "
+                             "automatically.")
         params: ScenarioBaseParams = cls.get_params_cls().from_dict(d)
         params.model_params = cls.model_cls().get_params_cls().from_dict(d['model_params'])
         params.data_params = cls.data_cls().get_params_cls().from_dict(d['data_params'])
+        params.evaluator_params = cls.evaluator_cls().default_params().__class__.from_dict(d.get('evaluator_params', {}))
         return params
 
     @classmethod
-    def from_path(cls, path: str) -> Tuple[Type['ScenarioBase'], ScenarioBaseParams]:
-        trainer_params_json_path = os.path.join(path, 'trainer_params.json')
-        scenario_params_json_path = os.path.join(path, 'scenario_params.json')
-        if os.path.exists(trainer_params_json_path):
-            with open(trainer_params_json_path) as f:
-                scenario_params_dict = json.load(f)['scenario_params']
-        elif os.path.exists(scenario_params_json_path):
-            with open(scenario_params_json_path) as f:
-                scenario_params_dict = json.load(f)
-        else:
-            raise FileNotFoundError(f"Either {trainer_params_json_path} or {scenario_params_json_path} must exist!")
+    def params_from_path(cls, path: str) -> ScenarioBaseParams:
+        if cls == ScenarioBase:
+            raise ValueError("You are calling this method from the ScenarioBaseClass. This is not supported. "
+                             "Call this method from an actual Scenario or call from_path to obtain the real class "
+                             "automatically.")
+        return cls.params_from_dict(cls.read_params_dict_from_path(path))
 
-        scenario_params_dict['data_params']['resource_base_path'] = path
-        return cls.from_dict(scenario_params_dict)
+    @classmethod
+    def read_params_dict_from_path(cls, path: str) -> dict:
+        if os.path.isfile(path):
+            # already a json file
+            with open(path) as f:
+                d = json.load(f)
+                if 'scenario_params' in d:
+                    scenario_params_dict = d['scenario_params']
+                else:
+                    scenario_params_dict = d
+        else:
+            trainer_params_json_path = os.path.join(path, 'trainer_params.json')
+            scenario_params_json_path = os.path.join(path, 'scenario_params.json')
+            if os.path.exists(trainer_params_json_path):
+                with open(trainer_params_json_path) as f:
+                    scenario_params_dict = json.load(f)['scenario_params']
+            elif os.path.exists(scenario_params_json_path):
+                with open(scenario_params_json_path) as f:
+                    scenario_params_dict = json.load(f)
+            else:
+                raise FileNotFoundError(f"Either {trainer_params_json_path} or {scenario_params_json_path} must exist!")
+
+        scenario_params_dict['data_params']['resource_base_path_'] = path
+        return scenario_params_dict
+
+    @classmethod
+    def trainer_params_from_dict(cls, d: dict) -> 'TrainerParams':
+        if cls == ScenarioBase:
+            raise ValueError("You are calling this method from the ScenarioBaseClass. This is not supported. "
+                             "Call this method from an actual Scenario or call from_dict to obtain the real class "
+                             "automatically.")
+        scenario_params = cls.params_from_dict(d['scenario_params'])
+        trainer_params = cls.trainer_cls().get_params_cls().from_dict(d)
+        trainer_params.scenario_params = scenario_params
+        return trainer_params
+
+    @classmethod
+    def from_path(cls, path: str) -> Tuple[Type['ScenarioBase'], ScenarioBaseParams]:
+        if cls != ScenarioBase:
+            raise ValueError("You are calling this method from a real Scenario class. Call params_from_path instead")
+        return cls.from_dict(cls.read_params_dict_from_path(path))
 
     @classmethod
     def from_dict(cls, d: dict) -> Tuple[Type['ScenarioBase'], ScenarioBaseParams]:
+        if cls != ScenarioBase:
+            raise ValueError("You are calling this method from a real Scenario class. Call params_from_dict instead")
         scenario_params = ScenarioBaseParams.from_dict(d)
         spec = importlib.util.spec_from_file_location(scenario_params.scenario_module_,
                                                       scenario_params.scenario_base_path_)
@@ -150,40 +202,88 @@ class ScenarioBase(ABC):
     @classmethod
     @abstractmethod
     def data_cls(cls) -> Type['DataBase']:
-        raise NotImplemented
+        raise NotImplementedError
 
     @classmethod
     @abstractmethod
     def model_cls(cls) -> Type['ModelBase']:
-        raise NotImplemented
+        raise NotImplementedError
 
     @classmethod
     def trainer_cls(cls) -> Type['Trainer']:
-        from tfaip.base.trainer import Trainer
+        from tfaip.base.imports import Trainer
         return Trainer
 
     @classmethod
     def lav_cls(cls) -> Type['LAV']:
-        from tfaip.base.lav import LAV
+        from tfaip.base.imports import LAV
         return LAV
 
+    @classmethod
+    def multi_lav_cls(cls) -> Type['MultiLAV']:
+        from tfaip.base.imports import MultiLAV
+        return MultiLAV
+
     @classmethod
     def predictor_cls(cls) -> Type['Predictor']:
-        from tfaip.base.predict import Predictor
+        from tfaip.base.imports import Predictor
         return Predictor
 
+    @classmethod
+    def evaluator_cls(cls) -> Type['Evaluator']:
+        return Evaluator
+
+    @classmethod
+    def multi_predictor_cls(cls) -> Type['MultiModelPredictor']:
+        from tfaip.base.predict.multimodelpredictor import MultiModelPredictor
+        return MultiModelPredictor
+
     @classmethod
     def create_trainer(cls, trainer_params: 'TrainerParams', restore=False) -> 'Trainer':
         return cls.trainer_cls()(trainer_params, cls(trainer_params.scenario_params), restore)
 
     @classmethod
     def create_lav(cls, lav_params: 'LAVParams', scenario_params: 'ScenarioBaseParams') -> 'LAV':
-        return cls.lav_cls()(lav_params, lambda: cls.data_cls()(scenario_params.data_params),
-                             lambda: cls.model_cls()(scenario_params.model_params))
+        return cls.lav_cls()(lav_params,
+                             data_fn=lambda: cls.data_cls()(scenario_params.data_params),
+                             model_fn=lambda: cls.model_cls()(scenario_params.model_params),
+                             predictor_fn=cls.predictor_cls(),
+                             evaluator_fn=lambda: cls.create_evaluator(scenario_params.evaluator_params),
+                             )
 
     @classmethod
-    def create_predictor(cls, params: 'PredictorParams', scenario_params: 'ScenarioBaseParams') -> 'Predictor':
-        return cls.predictor_cls()(params, cls.model_cls()(scenario_params.model_params), cls.data_cls()(scenario_params.data_params))
+    def create_multi_lav(cls, lav_params: 'LAVParams', scenario_params: 'ScenarioBaseParams', predictor_params: Optional['PredictorParams']=None):
+        return MultiLAV(
+            lav_params,
+            scenario_params.data_params.val,
+            cls.create_multi_predictor,
+            cls.create_evaluator(scenario_params.evaluator_params),
+            predictor_params=predictor_params if predictor_params is not None else cls.multi_predictor_cls().get_params_cls()(),
+        )
+
+    @classmethod
+    def create_predictor(cls, model: str, params: 'PredictorParams') -> 'Predictor':
+        data_params = cls.params_from_path(model).data_params
+        predictor = cls.predictor_cls()(params, cls.data_cls()(data_params))
+        if isinstance(model, str):
+            model = keras.models.load_model(os.path.join(model, 'serve'),
+                                            compile=False,
+                                            custom_objects=cls.model_cls().get_all_custom_objects(),
+                                            )
+
+        predictor.set_model(model)
+        return predictor
+
+    @classmethod
+    def create_multi_predictor(cls, paths: List[str], params: 'PredictorParams') -> 'MultiModelPredictor':
+        predictor_cls = cls.multi_predictor_cls()
+        return predictor_cls.from_paths(paths, params, cls)
+
+    @classmethod
+    def create_evaluator(cls, params: EvaluatorParams) -> Evaluator:
+        if cls.evaluator_cls() is None:
+            raise NotImplementedError
+        return cls.evaluator_cls()(params)
 
     @staticmethod
     def get_params_cls() -> Type[ScenarioBaseParams]:
@@ -201,7 +301,7 @@ class ScenarioBase(ABC):
         self._params = params
 
         # Track the global step
-        self._keras_model_data = KerasModelData([], [])
+        self._keras_model_data = KerasModelData([], [], [])
         self._keras_train_model: keras.Model = None
         self._export_graphs: Dict[str, ExportGraph] = {}
         self._keras_predict_model: keras.Model = None
@@ -246,38 +346,41 @@ class ScenarioBase(ABC):
             self._export_resources(path, scenario_params_dict)
 
         # Export frozen model
-        full_model_func = tf.function(lambda x: self._keras_predict_model(x))
-        full_model_concrete = full_model_func.get_concrete_function(self._keras_predict_model.input)
-        # lower_control_flow=True enables tf1 compatibility by disabling tf2 control flow for ops like if/while
-        frozen_func = convert_variables_to_constants_v2(full_model_concrete, lower_control_flow=True)
-        frozen_func.graph.as_graph_def()
-        path_frozen = os.path.join(path, 'frozen')
-        os.makedirs(path_frozen, exist_ok=True)
-        id_frozen = 'frozen_model.pb'
-        tf.io.write_graph(graph_or_graph_def=frozen_func.graph,
-                          logdir=path_frozen,
-                          name=id_frozen,
-                          as_text=False)
+        if self._params.export_frozen:
+            full_model_func = tf.function(lambda x: self._keras_predict_model(x))
+            full_model_concrete = full_model_func.get_concrete_function(self._keras_predict_model.input)
+            # lower_control_flow=True enables tf1 compatibility by disabling tf2 control flow for ops like if/while
+            frozen_func = convert_variables_to_constants_v2(full_model_concrete, lower_control_flow=True)
+            frozen_func.graph.as_graph_def()
+            path_frozen = os.path.join(path, self._params.frozen_dir_)
+            os.makedirs(path_frozen, exist_ok=True)
+            tf.io.write_graph(graph_or_graph_def=frozen_func.graph,
+                              logdir=path_frozen,
+                              name=self._params.frozen_filename_,
+                              as_text=False)
 
         # Export serve models
-        for label, export_graph in self._export_graphs.items():
-            if label == 'default':
-                path_serve = os.path.join(path, 'serve')  # default model handled separately
-            else:
-                path_serve = os.path.join(path, 'additional', label)
-            os.makedirs(path_serve, exist_ok=True)
-            export_graph.model.save(path_serve, include_optimizer=False)
+        if self._params.export_serve:
+            for label, export_graph in self._export_graphs.items():
+                if label == 'default':
+                    path_serve = os.path.join(path, self._params.default_serve_dir_)  # default model handled separately
+                else:
+                    path_serve = os.path.join(path, self._params.additional_serve_dir_, label)
+                os.makedirs(os.path.dirname(path_serve), exist_ok=True)
+                export_graph.model.save(path_serve, include_optimizer=False)
 
-        with open(os.path.join(path, 'net_config.json'), 'w') as f:
-            json.dump(self.net_config().to_dict(), f, indent=2)
+
+        if self._params.export_net_config_:
+            with open(os.path.join(path, self._params.net_config_filename_), 'w') as f:
+                json.dump(self.net_config().to_dict(), f, indent=2)
 
         if trainer_params_dict:
-            params_path = os.path.join(path, 'trainer_params.json')
+            params_path = os.path.join(path, self._params.trainer_params_filename_)
             logger.debug("Storing trainer params to '{}'".format(params_path))
             with open(params_path, 'w') as f:
                 json.dump(trainer_params_dict, f, indent=2)
         else:
-            params_path = os.path.join(path, 'scenario_params.json')
+            params_path = os.path.join(path, self._params.scenario_params_filename_)
             with open(params_path, 'w') as f:
                 json.dump(scenario_params_dict, f, indent=2)
 
@@ -317,9 +420,9 @@ class ScenarioBase(ABC):
             logger.info("Debugging Graph Construction. Breakpoints during construction are supported")
             # process one example in KerasDebugModel which builds the graph based on this example
             keras_debug_model = KerasDebugModel(self.model)
-            with self.data:
+            with self.data.get_train_data() as train_data:
                 out = keras_debug_model.predict(
-                    self._wrapped_train_data(steps_per_epoch=1).take(self._params.debug_graph_n_examples))
+                    self._wrap_data(train_data.input_dataset(), steps_per_epoch=1, is_debug_data=True).take(self._params.debug_graph_n_examples))
             logger.info("Mean values of debug model output: {}".format({k: v.mean() for k, v in out.items()}))
 
         # This regroups all inputs/targets as input to allow to access the complete data during training
@@ -349,12 +452,16 @@ class ScenarioBase(ABC):
         logger.info("Building prediction graph")
         pred_outputs = self.model.build(real_inputs)
         # rename outputs to dict keys (for export)
-        pred_outputs = {k: v if v.op.name == k else tf.identity(v, name=k) for k, v in pred_outputs.items()}
-        for k, v in pred_outputs.items():
-            if v.op.name != k:
-                raise NameError(
-                    f"Name of output operation {v.op.name} could not be set to {k} in the prediction graph. "
-                    f"This usually happens if the same name is used twice in a graph.")
+        # TODO: Check if renaming is still required, this is not possible in tf 2.4
+        if tf.version.VERSION >= "2.4.0":
+            pass
+        else:
+            pred_outputs = {k: v if v.op.name == k else tf.identity(v, name=k) for k, v in pred_outputs.items()}
+            for k, v in pred_outputs.items():
+                if v.op.name != k:
+                    raise NameError(
+                        f"Name of output operation {v.op.name} could not be set to {k} in the prediction graph. "
+                        f"This usually happens if the same name is used twice in a graph.")
 
         # inject the evaluate layer to the first output.
         # Note, if the first output is not used in the graph, nothing will be printed
@@ -374,10 +481,12 @@ class ScenarioBase(ABC):
             **real_outputs, **_losses, **_extended_metrics, **_additional_outputs,
             **{k: extended_outputs[v.output] for k, v in _simple_metrics.items()}
         }
+        _tensorboard_outputs = self.model.tensorboard_handler.setup(real_inputs, real_outputs)
 
         self._keras_model_data = KerasModelData(
             list(_losses.keys()),
             list(_extended_metrics.keys()),
+            list(_tensorboard_outputs.keys()),
         )
 
         # create the model (and a second one for exporting)
@@ -398,10 +507,16 @@ class ScenarioBase(ABC):
             return p
 
         logger.info("Compiling training model including optimization")
+        loss_weights = self.model.loss_weights()
+        if loss_weights is not None:
+            for k in loss_weights.keys():
+                if k not in _losses:
+                    raise KeyError(f"Loss weight {k} specified but not used in losses (available {list(_losses.keys())}).")
         self._keras_train_model.compile(optimizer=optimizer,
                                         loss={k: wrap_loss for k, _ in _losses.items()},
                                         loss_weights=self.model.loss_weights(),
-                                        weighted_metrics={**{k: metric for k, _ in _extended_metrics.items()},
+                                        metrics={k: OutputHolderMetricWrapper(v.shape, v.dtype, self._params.tensorboard_logger_history_size, name=k) for k, v in _tensorboard_outputs.items()},
+                                        weighted_metrics={**{k: MeanMetricWrapper(lambda t, p: p, name=k) for k, _ in _extended_metrics.items()},
                                                           **{k: v.metric for k, v in _simple_metrics.items()}},
                                         run_eagerly=run_eagerly,
                                         )
@@ -418,36 +533,41 @@ class ScenarioBase(ABC):
                 keras.models.load_model(tmp)
             logger.info("Model can be successfully loaded")
 
-    def _wrap_data(self, dataset, steps_per_epoch, batch_size):
+    def _wrap_data(self, dataset, steps_per_epoch, is_debug_data=False):
+        if dataset is None:
+            return None
+
         # wrapper for model fit (allows for other arguments)
-        def regroup(inputs, targets):
+        def regroup(inputs: Dict[str, AnyTensor], targets: Dict[str, AnyTensor]):
+            batch_size = tf.shape(next(iter(inputs.values() if len(inputs) > 0 else targets.values())))[0]
             # see setup_training
             # regroup data for training into all as input, and only loss and metrics as output
             # this is required to allow for custom losses with multiple inputs
-            ones_batched = [1] * batch_size
+            zeros = tf.repeat(tf.constant(0, dtype=tf.int64), batch_size)
             if self._keras_train_model:
-                step_epoch = {'step': ones_batched * self._keras_train_model.optimizer.iterations,
-                              'epoch': ones_batched * self._keras_train_model.optimizer.iterations // steps_per_epoch}
+                step_epoch = {'step': zeros + self._keras_train_model.optimizer.iterations,
+                              'epoch': zeros + self._keras_train_model.optimizer.iterations // steps_per_epoch}
             else:
                 # No train model exists, this happens on model debug
-                step_epoch = {'step': ones_batched * 0,
-                              'epoch': ones_batched * 0}
+                step_epoch = {'step': zeros,
+                              'epoch': zeros}
             wrapped_inputs = {**inputs, **targets, **step_epoch}
-            wrapped_targets = {**{l: [0] * batch_size for l in
-                                  self._keras_model_data.loss_names + self._keras_model_data.extended_metric_names},
+            wrapped_targets = {**{l: zeros for l in
+                                  self._keras_model_data.loss_names + self._keras_model_data.extended_metric_names + self._keras_model_data.tensorboard_output_names},
                                **{k: targets[v.target] for k, v in self.model.metric().items() if v.target in targets}
                                }
-            wrapped_weights = self.model.sample_weights(inputs, targets)
+            if is_debug_data:
+                wrapped_weights = None
+            else:
+                wrapped_weights = self.model.sample_weights(inputs, targets)
+                for k in wrapped_weights.keys():
+                    if k not in wrapped_targets.keys():
+                        raise KeyError(f"Sample weight {k} given but not found in targets (metric). "
+                                       f"Available: {list(wrapped_targets.keys())}")
             return wrapped_inputs, wrapped_targets, wrapped_weights
 
         return dataset.map(regroup)
 
-    def _wrapped_train_data(self, steps_per_epoch):
-        return self._wrap_data(self.data.get_train_data(), steps_per_epoch, self._params.data_params.train_batch_size)
-
-    def _wrapped_val_data(self, steps_per_epoch):
-        return self._wrap_data(self.data.get_val_data(), steps_per_epoch, self._params.data_params.val_batch_size)
-
     def fit(self,
             epochs,
             callbacks,
@@ -456,13 +576,21 @@ class ScenarioBase(ABC):
             **kwargs):
         self._keras_train_model.summary(print_fn=logger.info)
 
-        with self.data:
-            self._keras_train_model.fit(self._wrapped_train_data(steps_per_epoch),
+        with ExitStack() as stack:
+            train_dataset = stack.enter_context(self.data.get_train_data()).input_dataset()
+            try:
+                val_dataset = stack.enter_context(self.data.get_val_data()).input_dataset()
+            except ValueError:
+                logger.warning("Training without validation.")
+                # No val dataset available
+                val_dataset = None
+
+            self._keras_train_model.fit(self._wrap_data(train_dataset, steps_per_epoch),
                                         epochs=epochs,
                                         callbacks=callbacks,
                                         steps_per_epoch=steps_per_epoch,
                                         initial_epoch=initial_epoch,
-                                        validation_data=self._wrapped_val_data(steps_per_epoch),
+                                        validation_data=self._wrap_data(val_dataset, steps_per_epoch),
                                         shuffle=False,
                                         **kwargs
                                         )
