@@ -16,13 +16,14 @@
 # tfaip. If not, see http://www.gnu.org/licenses/.
 # ==============================================================================
 from dataclasses import dataclass, field
-from typing import List, Dict
+from typing import List, Dict, Optional
 
 from dataclasses_json import dataclass_json, config, LetterCase
 
-from tfaip.base.data.data_base_params import DataBaseParams
-from tfaip.base.model import ModelBaseParams
-from tfaip.util.argument_parser import dc_meta
+from tfaip.base.data.databaseparams import DataBaseParams
+from tfaip.base.evaluator.params import EvaluatorParams
+from tfaip.base.model.modelbaseparams import ModelBaseParams
+from tfaip.util.argumentparser import dc_meta
 from tfaip.util.versioning import get_commit_hash
 from tfaip import __version__
 
@@ -47,12 +48,34 @@ class ScenarioBaseParams:
         help="Number of evaluation examples to print per evaluation, use -1 to print all"
     ))
 
-    model_params: ModelBaseParams = field(default_factory=lambda: ModelBaseParams())
-    data_params: DataBaseParams = field(default_factory=lambda: DataBaseParams())
+    tensorboard_logger_history_size: int = field(default=5, metadata=dc_meta(
+        help="Number of instances to store for outputing into tensorboard. Default (last n=5)"
+    ))
 
-    scenario_base_path_: str = None
-    scenario_module_: str = None
-    id_: str = None
+    export_frozen: bool = field(default=False, metadata=dc_meta(
+        help="Export the frozen graph alongside the saved model"
+    ))
+    export_serve: bool = field(default=True, metadata=dc_meta(
+        help="Export the serving model (saved model format)"
+    ))
+
+    model_params: ModelBaseParams = field(default_factory=ModelBaseParams)
+    data_params: DataBaseParams = field(default_factory=DataBaseParams)
+    evaluator_params: EvaluatorParams = field(default_factory=EvaluatorParams)
+
+    # Additional export params
+    export_net_config_: bool = True
+    net_config_filename_: str = 'net_config.json'
+    frozen_dir_: str = 'frozen'
+    frozen_filename_: str = 'frozen_model.pb'
+    default_serve_dir_: str = 'serve'
+    additional_serve_dir_: str = 'additional'
+    trainer_params_filename_: str = 'trainer_params.json'
+    scenario_params_filename_: str = 'scenario_params.json'
+
+    scenario_base_path_: Optional[str] = None
+    scenario_module_: Optional[str] = None
+    id_: Optional[str] = None
 
     tfaip_commit_hash_: str = field(default_factory=get_commit_hash)
     tfaip_version_: str = __version__
