@@ -20,24 +20,26 @@ from abc import ABC
 import logging
 import os
 from datetime import datetime
-from typing import Type, Tuple, Union
+from typing import Type, Tuple, Union, Callable
 import tensorflow as tf
-from tensorflow.python.keras.callbacks import ProgbarLogger
+from tfaip.base.trainer.callbacks.extract_logs import ExtractLogsCallback
 from typeguard import typechecked
 
 from tfaip.base.device_config import DeviceConfig, distribute_strategy
-from tfaip.base.scenario import ScenarioBase
+from tfaip.base.scenario.scenariobase import ScenarioBase
 from tfaip.base.trainer.callbacks.benchmark_callback import BenchmarkCallback
 from tfaip.base.trainer.callbacks.ema_callback import EMACallback
-from tfaip.base.trainer.callbacks.export_best import ExportBestCallback
+from tfaip.base.trainer.callbacks.earlystopping.callback import EarlyStoppingCallback
 from tfaip.base.trainer.callbacks.lav_callback import LAVCallback
 from tfaip.base.trainer.callbacks.logger_callback import LoggerCallback
+from tfaip.base.trainer.callbacks.progbar import TFAIPProgbarLogger
 from tfaip.base.trainer.callbacks.tensor_board_callback import TensorBoardCallback
 from tfaip.base.trainer.callbacks.tensorflow_fix import TensorflowFix
 from tfaip.base.trainer.callbacks.train_params_logger import TrainParamsLoggerCallback
 from tfaip.base.trainer.callbacks.fix_metric_labels import FixMetricLabelsCallback
 from tfaip.base.trainer.optimizer.gradient_accumulation_optimizer import create_gradient_accumulation_optimizer
-from tfaip.base.trainer.trainerparams import TrainerParams
+from tfaip.base.trainer.params import TrainerParams
+from tfaip.base.trainer.warmstart.warmstart_params import WarmstartParams
 from tfaip.base.trainer.warmstart.warmstarter import Warmstarter
 from tfaip.base.trainer.optimizer.weights_moving_average import WeightsMovingAverage
 from tfaip.util.random import set_global_random_seed
@@ -56,19 +58,25 @@ class Trainer(ABC):
         return TrainerParams
 
     @staticmethod
-    def restore_trainer(checkpoint: Union[str, dict]) -> 'Trainer':
-        if isinstance(checkpoint, str):
-            with open(os.path.join(checkpoint, 'trainer_params.json')) as f:
-                d = json.load(f)
-        else:
-            d = checkpoint
+    def parse_trainer_params(d: Union[str, dict]) -> Tuple['TrainerParams', Type[ScenarioBase]]:
+        if isinstance(d, str):
+            if not d.endswith('.json'):
+                d = os.path.join(d, 'trainer_params.json')
 
-        trainer_params: TrainerParams = TrainerParams.from_dict(d)
-        logger.info("trainer_params=" + trainer_params.to_json(indent=2))
+            with open(d) as f:
+                d = json.load(f)
         scenario, scenario_params = ScenarioBase.from_dict(d['scenario_params'])
+        trainer_params: TrainerParams = scenario.trainer_cls().get_params_cls().from_dict(d)
+        logger.info("trainer_params=" + trainer_params.to_json(indent=2))
 
         # Load the actual scenario params for the particular scenario
         trainer_params.scenario_params = scenario_params
+        return trainer_params, scenario
+
+    @classmethod
+    def restore_trainer(cls, checkpoint: Union[str, dict]) -> 'Trainer':
+        trainer_params, scenario = cls.parse_trainer_params(checkpoint)
+        logger.info("trainer_params=" + trainer_params.to_json(indent=2))
         trainer = scenario.create_trainer(trainer_params, restore=True)
         return trainer
 
@@ -100,20 +108,18 @@ class Trainer(ABC):
         if self._params.force_eager:
             tf.config.run_functions_eagerly(run_eagerly=True)
 
-        self._warmstarter = Warmstarter(self._params.warmstart_params)
         self._scenario = scenario
         scenario.setup()
         self._data = scenario.data
         self._model = scenario.model
         self.stop_training = False
-
-        self._steps_per_epoch = self._params.samples_per_epoch // self._data.params().train_batch_size
-        if self._steps_per_epoch <= 0:
-            raise ValueError(f"Samples per epoch must be greater than the train batch size, but got "
-                             f"{self._params.samples_per_epoch} < {self._data.params().train_batch_size}")
+        self._steps_per_epoch: int = None   # Not initialized yet
+        self._callbacks = []
 
         os.environ['TF_CPP_MIN_LOG_LEVEL'] = str(self._params.tf_cpp_min_log_level)
 
+
+
     @property
     def scenario(self):
         return self._scenario
@@ -123,9 +129,11 @@ class Trainer(ABC):
         return self._params
 
     @distribute_strategy
-    def train(self, callbacks=None):
-        external_callbacks = callbacks
-        callbacks = []
+    def train(self,
+              callbacks=None,
+              warmstart_fn: Callable[[WarmstartParams], Warmstarter] = Warmstarter,
+              ):
+        self.setup_steps_per_epoch()
 
         self._params.learning_rate_params.epochs_ = self._params.epochs
         self._params.learning_rate_params.steps_per_epoch_ = self._steps_per_epoch
@@ -138,47 +146,14 @@ class Trainer(ABC):
             logger.info("Restoring from checkpoint '{}'".format(self._params.checkpoint_dir))
             # load_weights also restores the optimizer weights!
             self._scenario.keras_train_model.load_weights(
-                os.path.join(self._params.checkpoint_dir, 'variables', 'variables'))
+                os.path.join(self._params.checkpoint_dir, self._params.saved_checkpoint_sub_dir_, 'variables', 'variables'))
             if self._params.warmstart_params.model:
                 logger.warning("Ignoring warmstart since training is resumed from a checkpoint")
         else:
-            self._warmstarter.warmstart(self._scenario.keras_train_model)
+            custom_objects = self._model.__class__.get_all_custom_objects()
+            warmstart_fn(self.params.warmstart_params).warmstart(self._scenario.keras_train_model, custom_objects)
 
-        callbacks.append(FixMetricLabelsCallback())
-        callbacks.append(ProgbarLogger(count_mode='steps'))  # Progbar after label fix
-        callbacks.append(TensorflowFix())
-        callbacks.append(BenchmarkCallback())
-
-        if self._params.lav_every_n >= 1:
-            # LAV callback must be assigned before export best to allow to export based on best LAV
-            callbacks.append(LAVCallback(self._params, self._scenario))
-
-        if self._params.checkpoint_dir:
-            if self._params.write_checkpoints:
-                # we need only weights to restore the training, the graph will be reconstructed
-                variables_path = os.path.join(self._params.checkpoint_dir, 'variables', 'variables')
-                callbacks.append(tf.keras.callbacks.ModelCheckpoint(variables_path, save_weights_only=True))
-            callbacks.append(TrainParamsLoggerCallback(self._params, self._params.checkpoint_dir))
-
-        if self._params.calc_ema:
-            # EMA must be before export best to export ema
-            # noinspection PyTypeChecker
-            callbacks.append(EMACallback(optimizer))
-
-        if self._params.export_best and self._params.checkpoint_dir:
-            callbacks.append(
-                ExportBestCallback(os.path.join(self._params.checkpoint_dir, 'best'), self._scenario, self._params))
-
-        if self._params.checkpoint_dir:
-            # Tensorflow Callback as last, so that it is allowed to add additional outputs (e.g. LAVCallback)
-            callbacks.append(TensorBoardCallback(log_dir=self._params.checkpoint_dir,
-                                                 steps_per_epoch=self._steps_per_epoch,
-                                                 reset=self._params.current_epoch == 0,
-                                                 profile="10,20" if self._params.profile else 0))
-
-        callbacks.append(LoggerCallback())
-        if external_callbacks:
-            callbacks.extend(external_callbacks)
+        callbacks = self.setup_callbacks(optimizer, callbacks)
 
         if self._params.epochs <= self._params.current_epoch:
             logger.warning(
@@ -192,17 +167,111 @@ class Trainer(ABC):
                 # Set fixed random seed for training if desired, this makes training independent of previous operations
                 # such as loading/creating model from scratch
                 set_global_random_seed(self._params.random_seed + 1)
-            self._scenario.fit(epochs=self._params.epochs,
-                               initial_epoch=self._params.current_epoch,
-                               steps_per_epoch=self._steps_per_epoch,
-                               callbacks=callbacks,
-                               )
+
+            self._callbacks = callbacks
+            self.fit()
 
         # export the model to "checkpoint_dir/export"
         if self._params.checkpoint_dir and self._params.export_final:
             logger.info("Final export of the model.")
             self._scenario.export(os.path.join(self._params.checkpoint_dir, 'export'))
 
+
+    def create_train_params_logger_callback(self):
+        if self._params.checkpoint_dir:
+            save_freq = self._params.checkpoint_save_freq_
+            if self._params.write_checkpoints:
+                # we need only weights to restore the training, the graph will be reconstructed
+                variables_path = os.path.join(self._params.checkpoint_dir, self._params.checkpoint_sub_dir_,
+                                              'variables', 'variables')
+                if isinstance(save_freq, str) and save_freq.isdigit():
+                    save_freq = int(save_freq)
+                if isinstance(save_freq, int):
+                    save_freq = save_freq * self._steps_per_epoch
+                if save_freq == 0:
+                    save_freq = None
+        else:
+            save_freq = None
+
+        return TrainParamsLoggerCallback(self._params, save_freq)
+
+    def setup_callbacks(self,
+                        optimizer,
+                        callbacks=None,
+                        ):
+        external_callbacks = callbacks
+        callbacks = []
+        tensorboard_data_handler = self._model.tensorboard_handler
+
+        callbacks.append(FixMetricLabelsCallback())
+        extract_logs_cb = ExtractLogsCallback(tensorboard_data_handler)
+        callbacks.append(extract_logs_cb)
+        callbacks.append(TFAIPProgbarLogger(delta_time=self._params.progbar_delta_time, count_mode='steps'))
+        callbacks.append(TensorflowFix())
+        callbacks.append(BenchmarkCallback())
+
+        if self._params.lav_every_n >= 1:
+            # LAV callback must be assigned before export best to allow to export based on best LAV
+            callbacks.append(LAVCallback(self._params, self._scenario))
+
+        callbacks.append(self.create_train_params_logger_callback())
+
+        if self._params.calc_ema:
+            # EMA must be before export best to export ema
+            # noinspection PyTypeChecker
+            callbacks.append(EMACallback(optimizer))
+
+        callbacks.append(EarlyStoppingCallback(self._scenario, self._params))
+
+        if self._params.checkpoint_dir:
+            # Tensorflow Callback as last, so that it is allowed to add additional outputs (e.g. LAVCallback)
+            callbacks.append(TensorBoardCallback(log_dir=self._params.checkpoint_dir,
+                                                 steps_per_epoch=self._steps_per_epoch,
+                                                 extracted_logs_cb=extract_logs_cb,
+                                                 data_handler=tensorboard_data_handler,
+                                                 reset=self._params.current_epoch == 0,
+                                                 profile="10,20" if self._params.profile else 0))
+
+        callbacks.append(LoggerCallback())
+        if external_callbacks:
+            callbacks.extend(external_callbacks)
+
+        return callbacks
+
+    def setup_steps_per_epoch(self):
+        if self._params.samples_per_epoch < 0:
+            logger.info(f"Setting samples per epoch relative to dataset size with a factor of "
+                        f"{self._params.scale_epoch_size}. Note that this "
+                        "requires the creation of the data generator once before training.")
+            samples_per_epoch = len(self._data.get_train_data().create_data_generator())
+
+            if self._params.scale_epoch_size != 1:
+                samples_per_epoch = int(samples_per_epoch * self._params.scale_epoch_size)
+
+            if samples_per_epoch <= 0:
+                raise ValueError("Could not compute the number of samples per epoch based on the size of the data "
+                                 "generator. Please implement __len__ correctly.")
+            logger.info(f"Set samples per epoch to {samples_per_epoch}")
+        else:
+            samples_per_epoch = self._params.samples_per_epoch
+            if self._params.scale_epoch_size != 1:
+                logger.warning("Setting scale_epoch_size has no effect when using absolute values for samples_per_epoch."
+                               "Set samples_per_epoch to the default (=-1) to use relative computation.")
+
+        self._steps_per_epoch = samples_per_epoch // self._data.params().train.batch_size
+        if self._steps_per_epoch <= 0:
+            raise ValueError(f"Samples per epoch must be greater than the train batch size, but got "
+                             f"{samples_per_epoch} < {self._data.params().train.batch_size}")
+
+    def fit(self):
+        self._scenario.fit(epochs=self._params.epochs,
+                           initial_epoch=self._params.current_epoch,
+                           steps_per_epoch=self._steps_per_epoch,
+                           validation_freq=self._params.val_every_n,
+                           callbacks=self._callbacks,
+                           verbose=self._params.verbose,
+                           )
+
     @typechecked
     def _create_optimizer(self) -> tf.keras.optimizers.Optimizer:
         # Create the optimizer
@@ -218,6 +287,18 @@ class Trainer(ABC):
                 "clipnorm": clip_grad if clip_grad > 0 else None,
                 "clipvalue": -clip_grad if clip_grad < 0 else None,
             }
+            if self._params.optimizer_params.optimizer == 'SGD':
+                args['momentum'] = self._params.optimizer_params.momentum
+            elif self._params.optimizer_params.optimizer in ['Adam', 'Adamax']:
+                args['beta_1'] = self._params.optimizer_params.beta_1
+                args['beta_2'] = self._params.optimizer_params.beta_2
+                args['epsilon'] = self._params.optimizer_params.epsilon
+            elif self._params.optimizer_params.optimizer in ['RMSprop']:
+                args['momentum'] = self._params.optimizer_params.momentum
+                args['rho'] = self._params.optimizer_params.rho
+                args['centered'] = self._params.optimizer_params.centered
+                args['epsilon'] = self._params.optimizer_params.epsilon
+
             if self._params.calc_ema:
                 return WeightsMovingAverage, {'optimizer': real_optimizer(**args)}
             else:
