@@ -24,7 +24,7 @@ import importlib
 
 
 if TYPE_CHECKING:
-    from tfaip.base.scenario import ScenarioBase
+    from tfaip.base.imports import ScenarioBase
 
 
 @dataclass
@@ -32,6 +32,7 @@ class ScenarioDefinition:
     path: str
     name: str
     scenario: Type['ScenarioBase']
+    module: str
 
 
 _scenarios: List[ScenarioDefinition] = []
@@ -63,7 +64,8 @@ def load_all_scenarios(dir_name, root_package='', root_dir=None):
                     register_scenario(ScenarioDefinition(
                         dir_name,
                         os.path.relpath(dir_name, root_dir).replace("/", '.'),
-                        scenario_cls
+                        scenario_cls,
+                        scenario_cls.__module__,
                     ))
 
         load_all_scenarios(os.path.join(dir_name, package_name), '.'.join([root_package, package_name]), root_dir)
@@ -74,11 +76,25 @@ def scenarios():
         load_all_scenarios(this_dir, 'tfaip.scenario')
 
         # load external scenarios
-        external_dirs = os.environ.get('TFAIP_SCENARIOS', '').split(';')
+        external_dirs = os.environ.get('TFAIP_SCENARIOS', '').split(':')
         for external_dir in external_dirs:
             if len(external_dir) == 0:
                 continue
             logger.info(f"Loading external scenarios from {external_dir}")
-            load_all_scenarios(external_dir, '', root_dir=external_dir)
+            load_all_scenarios(external_dir, os.path.split(external_dir)[-1], root_dir=external_dir)
 
     return _scenarios
+
+
+def get_scenario_by_name(name: str) -> ScenarioDefinition:
+    for scenario in scenarios():
+        if scenario.name == name:
+            return scenario
+    raise IndexError(f"Scenario of name {name} not found. Available scenarios {', '.join(s.name for s in scenarios())}")
+
+
+def get_scenario_by_module(module: str) -> ScenarioDefinition:
+    for scenario in scenarios():
+        if scenario.module == module:
+            return scenario
+    raise IndexError(f"Scenario of module {module} not found. Available scenarios {', '.join(s.module for s in scenarios())}")
