@@ -17,6 +17,6 @@
 # ==============================================================================
 def read_utf8_textline_reference_from_file(fn: str) -> str:
     with open(fn, 'r') as ref_file:
-        aLine = ref_file.readline()
-        aLine = aLine.rstrip('\n')
-        return aLine
\ No newline at end of file
+        a_line = ref_file.readline()
+        a_line = a_line.rstrip('\n')
+        return a_line
