@@ -19,21 +19,39 @@ import os
 import logging
 
 # Initialize logging
+import sys
+
 FORMAT = '{levelname:<8s} {asctime} {name:>30.30s}: {message}'
 formatter = logging.Formatter(FORMAT, style='{')
-logger = logging.getLogger(__name__)
-logging.basicConfig(level=logging.INFO)
+TFAIP_LOG_LEVEL = getattr(logging, os.environ.get('TFAIP_LOG_LEVEL', 'INFO').upper())
+this_logger = logging.getLogger(__name__)
+logging.basicConfig(level=TFAIP_LOG_LEVEL)
 logging.getLogger().handlers[0].setFormatter(formatter)
 
 for handler in logging.getLogger("tensorflow").handlers:
     handler.setFormatter(formatter)
 
 
+def handle_exception(exc_type, exc_value, exc_traceback):
+    if issubclass(exc_type, KeyboardInterrupt):
+        sys.__excepthook__(exc_type, exc_value, exc_traceback)
+        return
+
+    this_logger.critical("Uncaught exception", exc_info=(exc_type, exc_value, exc_traceback))
+
+
+sys.excepthook = handle_exception
+
+
 def setup_log(log_dir, append, log_name='train.log'):
     os.makedirs(log_dir, exist_ok=True)
     filename = os.path.join(log_dir, log_name)
     file_handler = logging.FileHandler(filename, 'a' if append else 'w')
     file_handler.setFormatter(formatter)
-    file_handler.setLevel(level=logging.INFO)
+    file_handler.setLevel(level=TFAIP_LOG_LEVEL)
     logging.getLogger().addHandler(file_handler)
-    logger.info(f"Logging training progress to '{filename}'")
+    this_logger.info(f"Logging training progress to '{filename}'")
+
+
+def logger(name):
+    return logging.getLogger(name)
